
#ifndef VERIFYYAPN_TAPN_TIMEDPLACE_HPP_
#define VERIFYYAPN_TAPN_TIMEDPLACE_HPP_

#include <string>
#include <vector>
#include <iostream>
#include "TimeInvariant.hpp"
#include "TimedInputArc.hpp"
#include "OutputArc.hpp"
#include "TransportArc.hpp"
#include "InhibitorArc.hpp"
#include "boost/shared_ptr.hpp"

namespace VerifyTAPN{
	namespace TAPN{
		enum PlaceType {
			Inv, Dead, Std
		};

		class TimedPlace {
		public: // static
			static const TimedPlace& Bottom() {
				static TimedPlace bottom;
				return bottom;
			}

			static const int BottomIndex() {
				return -1;
			}

			static const std::string BOTTOM_NAME;

		public: // typedefs
			typedef std::vector< boost::shared_ptr<TimedPlace> > Vector;

		public: // construction / destruction
			TimedPlace(const std::string& name, const std::string& id, const TimeInvariant timeInvariant)
			: name(name), id(id), timeInvariant(timeInvariant), index(-2), untimed(false), maxConstant(0), containsInhibitorArcs(false), inputArcs(), transportArcs(), inhibitorArcs()  { };
			TimedPlace() : name(BOTTOM_NAME), timeInvariant(), index(BottomIndex()), untimed(false), maxConstant(0), containsInhibitorArcs(false), inputArcs(), transportArcs(), inhibitorArcs(){ };
			virtual ~TimedPlace() { /* empty */ };

		public: // modifiers
			inline void markPlaceAsUntimed() { untimed = true; }
			inline void setIndex(int i) { index = i; };
			inline void setMaxConstant(int max) { maxConstant = max; }
			inline void setHasInhibitorArcs(bool inhibitorArcs) { containsInhibitorArcs = inhibitorArcs; }
			inline void addInputArc(boost::shared_ptr<TimedInputArc> inputArc) { inputArcs.push_back(inputArc); }
			inline void addTransportArc(boost::shared_ptr<TransportArc> transportArc) { transportArcs.push_back(transportArc); }
			inline void addInhibitorArc(boost::shared_ptr<InhibitorArc> inhibitorArc) { inhibitorArcs.push_back(inhibitorArc); }
			inline void addOutputArc(boost::shared_ptr<OutputArc> outputArc) { outputArcs.push_back(outputArc); }
			inline void setType(PlaceType type){ this->type = type; }
		public: // inspection
			inline const PlaceType getType() const { return type; }
			const std::string& getName() const;
			const std::string& getId() const;
			void print(std::ostream& out) const;
			inline int getIndex() const { return index; };
			inline const bool isUntimed() const { return untimed; }
			inline const int getMaxConstant() const { return maxConstant; }
			inline const TAPN::TimeInvariant& getInvariant() const { return timeInvariant; };
			inline bool hasInhibitorArcs() const { return containsInhibitorArcs; };
			inline const TransportArc::WeakPtrVector& getTransportArcs() const { return transportArcs; }
			inline const InhibitorArc::WeakPtrVector& getInhibitorArcs() const { return inhibitorArcs; }
			inline const TimedInputArc::WeakPtrVector& getInputArcs() const { return inputArcs; }

		private: // data
			PlaceType type;
			std::string	name;
			std::string id;
			TimeInvariant timeInvariant;
			int index;
			bool untimed;
			int maxConstant;
			bool containsInhibitorArcs;
			TimedInputArc::WeakPtrVector inputArcs;
			TransportArc::WeakPtrVector transportArcs;
			InhibitorArc::WeakPtrVector inhibitorArcs;
			OutputArc::WeakPtrVector outputArcs;
		};

		inline std::ostream& operator<<(std::ostream& out, const TimedPlace& place)
		{
			place.print(out);
			return out;
		}

		// TAPAAL does not allow multiple places with the same name,
		// thus it is enough to use the name to determine equality.
		inline bool operator==(TimedPlace const& a, TimedPlace const& b)
		{
			return a.getName() == b.getName();
		}

		inline bool operator!=(TimedPlace const& a, TimedPlace const& b)
		{
			return !(a.getName() == b.getName());
		}
	}
}
#endif /* VERIFYYAPN_TAPN_TIMEDPLACE_HPP_ */
